@@ -47,7 +47,7 @@ crew_test("crew_worker() can run mirai tasks and assigns env vars", {
     instance = instance
   )
   crew_retry(
-    ~!nanonext::.unresolved(m$data),
+    ~!nanonext::unresolved(m$data),
     seconds_interval = 0.1,
     seconds_timeout = 5
   )
